﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;
using NetSDKCS;
using System.Diagnostics;
using System.Windows.Controls;
using Helper;
using System.IO;

namespace RealPlayDemo
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        MainWindow mainWindow; //main window object.
        fDisConnectCallBack disConnect; //call back for device disconnect.


        // override OnStartUp function for set NetClient init function and disconnect callback.
        // and exit to call cleanup function.
        protected override void OnStartup(StartupEventArgs e)
        {
            LogHelper.WriteWithMethod("Enter");
            base.OnStartup(e);
            LoadLanguage();
            mainWindow = new RealPlayDemo.MainWindow(); //instance mainwindow.
            disConnect = new fDisConnectCallBack(DisConnect); //set disconnect callback.
            try
            {
                bool res = NETClient.Init(disConnect,IntPtr.Zero, null); //init NetClient.
                if (res == false)
                {
                    LogHelper.WriteWithMethod(res.ToString());
                    MessageBox.Show("NETClient init failed!");
                }
                mainWindow.Show();
                LogHelper.WriteWithMethod("Leave");
            }
            catch (Exception ex)
            {
                LogHelper.WriteException(ex);
                MessageBox.Show(ex.Message);
                Process.GetCurrentProcess().Kill(); //general cannot find dll about netsdk or cannot call dll.
            }
        }

        /// <summary>
        /// load language
        /// </summary>
        private void LoadLanguage()
        {
            string path = AppDomain.CurrentDomain.BaseDirectory+"Language";
            string fileNamePath = string.Empty;
            ResourceDictionary resourceDictionary = new ResourceDictionary();
            switch (System.Globalization.CultureInfo.CurrentCulture.LCID)
            {
                case 0x00804:
                    fileNamePath = path + "\\zh-cn.xaml";
                    if (!File.Exists(fileNamePath))
                    {
                        resourceDictionary.Source = new Uri("pack://application:,,,/language/zh-cn.xaml");
                    }
                    else
                    {
                        resourceDictionary.Source = new Uri(fileNamePath, UriKind.RelativeOrAbsolute);
                    }                    
                    break;
                default:
                    fileNamePath = path + "\\en-us.xaml";
                    if (!File.Exists(fileNamePath))
                    {
                        resourceDictionary.Source = new Uri("pack://application:,,,/language/en-us.xaml");
                    }
                    else
                    {
                        resourceDictionary.Source = new Uri(fileNamePath, UriKind.RelativeOrAbsolute);
                    }
                    break;
            }
            App.Current.Resources.MergedDictionaries.Add(resourceDictionary);
        }

        //ovveride onexit function for call cleanup.
        protected override void OnExit(ExitEventArgs e)
        {
            LogHelper.WriteWithMethod("Enter");
            base.OnExit(e);
            NETClient.Cleanup();                        //cleanup NetClient resources.
            LogHelper.WriteWithMethod("Leave");
        }

        //device disconnect callback function.
        private void DisConnect(IntPtr lLoginID, IntPtr pchDVRIP, int nDVRPort, IntPtr dwUser)
        {
            LogHelper.WriteWithMethod("Enter");
            if (mainWindow == null)
            {
                return;
            }
            try
            {
                foreach (var item in mainWindow.m_DeviceList)
                {
                    if (lLoginID == item.LoginID)
                    {
                        //call delegate to update disconnect device information.here the thread cannot update UI,must call delegate or send message to update UI.
                        Dispatcher.BeginInvoke((Action<IntPtr>)UpdateTreeViewItem,item.LoginID);  
                    }
                }
            }
            catch(Exception ex)
            {
                LogHelper.WriteException(ex);
                MessageBox.Show(mainWindow, ex.Message);
            }
            LogHelper.WriteWithMethod("Leave");
        }
        //update treeviewitem information about disconnect device.
        private void UpdateTreeViewItem(IntPtr value)
        {
            LogHelper.WriteWithMethod("Enter");
            foreach (TreeViewItem treeItem in mainWindow.treeView.Items)
            {
                if ((treeItem.Tag as Device).LoginID== value)
                {
                    treeItem.Header = (treeItem.Tag as Device).IP + (string)App.Current.Resources["ID_OFFLINE"]; //set information to offline.
                }
            }
            LogHelper.WriteWithMethod("Leave");
        }
    }
}
